"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const lodash_1 = require("lodash");
const path_1 = __importDefault(require("path"));
const util_1 = require("util");
const dependencyPaths_1 = require("../helpers/dependencyPaths");
const PackageJSONConfig_1 = require("./PackageJSONConfig");
const Rearguard_1 = require("./Rearguard");
const dependenciesNotCreatedWithRearguard = new Set();
const dependenciesCreatedWithRearguard = new Set();
const exists = util_1.promisify(fs_1.default.exists);
const readdir = util_1.promisify(fs_1.default.readdir);
const readFile = util_1.promisify(fs_1.default.readFile);
class RearguardConfig extends PackageJSONConfig_1.PackageJSONConfig {
    getBin() {
        return this.getRearguard().bin;
    }
    getContext() {
        return this.getRearguard().webpack.context;
    }
    getModules() {
        return this.getRearguard().webpack.modules;
    }
    getEntry() {
        return this.getRearguard().webpack.entry;
    }
    getDllEntry() {
        return this.getRearguard().webpack.dll_entry;
    }
    getLibEntry() {
        return this.getRearguard().webpack.lib_entry;
    }
    getOutput() {
        return this.getRearguard().webpack.output;
    }
    getUnPublishedDependency() {
        return this.getRearguard().project.unPublishedDependency;
    }
    getComponents() {
        return this.getRearguard().project.components;
    }
    isPublishToGit() {
        return this.getRearguard().distribution.publish_to_git;
    }
    isOverwriteTSConfig() {
        return !this.getRearguard().configs.noOverwriteTSConfig;
    }
    isOverwriteTSTestConfig() {
        return !this.getRearguard().configs.noOverwriteTSTestConfig;
    }
    isOverwriteLintConfig() {
        return !this.getRearguard().configs.noOverwriteLintConfig;
    }
    isOverwriteGitIgnore() {
        return !this.getRearguard().configs.noOverwriteGitIgnore;
    }
    isLib() {
        const { type } = this.getRearguard().project;
        return type === "lib";
    }
    isApp() {
        const { type } = this.getRearguard().project;
        return type === "app";
    }
    isDll() {
        const { type } = this.getRearguard().project;
        return type === "dll";
    }
    isMono() {
        const { type } = this.getRearguard().project;
        return type === "mono";
    }
    isIsomorphic() {
        const { runtime } = this.getRearguard().project;
        return runtime === "isomorphic";
    }
    isBrowser() {
        const { runtime } = this.getRearguard().project;
        return runtime === "browser";
    }
    isNode() {
        const { runtime } = this.getRearguard().project;
        return runtime === "node";
    }
    isNodeLib() {
        return this.isNode() && this.isLib();
    }
    isNodeApp() {
        const { type } = this.getRearguard().project;
        return this.isNode() && type === "app";
    }
    willLoadOnDemand() {
        const { will_load_on_demand } = this.getRearguard().project;
        return will_load_on_demand;
    }
    getCSS() {
        const { postcssPlugins, useOnlyIsomorphicStyleLoader } = this.getRearguard().css;
        return [postcssPlugins, useOnlyIsomorphicStyleLoader];
    }
    async setRuntime(runtime) {
        await this.setRearguard(new Rearguard_1.Rearguard(lodash_1.merge(this.getRearguard(), { project: { runtime } })));
    }
    async setType(type) {
        await this.setRearguard(new Rearguard_1.Rearguard(lodash_1.merge(this.getRearguard(), { project: { type } })));
    }
    async setBin(bin) {
        await this.setRearguard(new Rearguard_1.Rearguard(lodash_1.merge(this.getRearguard(), { bin })));
    }
    async setComponents(components) {
        await this.setRearguard(new Rearguard_1.Rearguard(lodash_1.merge(this.getRearguard(), { project: { components } })));
    }
    async getDependenciesCreatedWithRearguard(monoDependencyDirs = [], searchInMonoDirectory = false) {
        const projectDeps = new Set(this.getUnPublishedDependency());
        if (searchInMonoDirectory) {
            for (const monoDependencyDir of monoDependencyDirs) {
                const pathToMonoDependency = path_1.default.resolve(this.CWD, monoDependencyDir);
                const dependencyDirs = (await readdir(pathToMonoDependency)).map((dirName) => path_1.default.resolve(pathToMonoDependency, dirName));
                for (const dependencyDir of dependencyDirs) {
                    const rearguardConfig = new RearguardConfig(dependencyDir);
                    const dependencyName = rearguardConfig.getName();
                    if (dependenciesCreatedWithRearguard.has(dependencyName)) {
                        projectDeps.add(dependencyName);
                        continue;
                    }
                    if (!dependenciesNotCreatedWithRearguard.has(dependencyName)) {
                        const pkgPath = path_1.default.resolve(dependencyDir, this.packageJsonFileName);
                        if (await RearguardConfig.isRearguard(pkgPath)) {
                            projectDeps.add(dependencyName);
                            dependenciesCreatedWithRearguard.add(dependencyName);
                        }
                        else {
                            dependenciesNotCreatedWithRearguard.add(dependencyName);
                        }
                    }
                }
            }
        }
        else {
            const nodeModulePath = this.findNodeModulesInParentDirectory(this.CWD);
            const dependencyList = this.getDependencyList();
            for (const dependencyName of dependencyList) {
                if (dependenciesCreatedWithRearguard.has(dependencyName)) {
                    projectDeps.add(dependencyName);
                    continue;
                }
                if (dependenciesNotCreatedWithRearguard.has(dependencyName)) {
                    continue;
                }
                const pkgPath = path_1.default.resolve(nodeModulePath, dependencyName, this.packageJsonFileName);
                if (await RearguardConfig.isRearguard(pkgPath)) {
                    projectDeps.add(dependencyName);
                    dependenciesCreatedWithRearguard.add(dependencyName);
                }
                else {
                    dependenciesNotCreatedWithRearguard.add(dependencyName);
                }
            }
        }
        return projectDeps;
    }
    findNodeModulesInParentDirectory(CWD, count = 0) {
        const nodeModules = dependencyPaths_1.getLocalNodeModulePath(CWD);
        if (fs_1.default.existsSync(nodeModules)) {
            return nodeModules;
        }
        else if (count <= 3) {
            return this.findNodeModulesInParentDirectory(path_1.default.resolve(CWD, ".."), count + 1);
        }
        else {
            throw new Error(`[ FIND_NODE_MODULES_IN_PARENT ][ node_modules not found here: ${nodeModules} ]`);
        }
    }
}
exports.RearguardConfig = RearguardConfig;
RearguardConfig.isRearguard = async (pkgPath) => {
    if (await exists(pkgPath)) {
        const pkgContent = await readFile(pkgPath, { encoding: "utf-8" });
        const pkg = JSON.parse(pkgContent);
        return pkg.hasOwnProperty("rearguard");
    }
    return false;
};
//# sourceMappingURL=RearguardConfig.js.map